@@ -1420,19 +1420,15 @@ yf::Zoom::BackendPtr yf::Zoom::Frontend::get_backend_from_databases(
         if (proxy.length())
             b->set_option("proxy", proxy);
     }
-    std::string url;
+    std::string url(sptr->target);
     if (sptr->sru.length())
     {
-        url = "http://" + sptr->target;
         b->set_option("sru", sptr->sru);
-
+        if (url.find("://") == std::string::npos)
+            url = "http://" + url;
         if (sptr->sru_version.length())
             b->set_option("sru_version", sptr->sru_version);
     }
-    else
-    {
-        url = sptr->target;
-    }
     if (no_out_args)
     {
         char *x_args = 0;
